@@ -17,13 +17,17 @@ module Database.Persist.GenericSql
     , Statement
     , runSqlConn
     , runSqlPool
-    , Key (..)
+    , Key
+
+    -- * Useful data types
+    , Checkmark(..)
 
     -- * Raw SQL queries
     -- $rawSql
     , rawSql
     , Entity(..)
     , Single(..)
+    , RawSql
 
     -- * Migrations
     , Migration
@@ -58,14 +62,17 @@ import Data.Text (Text, pack, unpack, concat)
 import qualified Data.Text as T
 import Web.PathPieces (PathPiece (..))
 import qualified Data.Text.Read
+import Data.Maybe (fromMaybe)
 import Data.Monoid (Monoid, mappend)
 import Database.Persist.EntityDef
 import qualified Data.Conduit as C
 import qualified Data.Conduit.List as CL
+import Control.Monad.Logger (MonadLogger)
+import Control.Monad.Base (liftBase)
 
 type ConnectionPool = Pool Connection
 
-instance PathPiece (Key SqlPersist entity) where
+instance PathPiece (KeyBackend R.SqlBackend entity) where
     toPathPiece (Key (PersistInt64 i)) = toPathPiece i
     toPathPiece k = throw $ PersistInvalidField $ "Invalid Key: " ++ show k
     fromPathPiece t =
@@ -73,39 +80,40 @@ instance PathPiece (Key SqlPersist entity) where
             Right (i, "") -> Just $ Key $ PersistInt64 i
             _ -> Nothing
 
-execute' :: MonadIO m => Text -> [PersistValue] -> SqlPersist m ()
+execute' :: (MonadIO m, MonadLogger m) => Text -> [PersistValue] -> SqlPersist m ()
 execute' = R.execute
 
 -- | Get a connection from the pool, run the given action, and then return the
 -- connection to the pool.
-runSqlPool :: (MonadBaseControl IO m, MonadIO m) => SqlPersist m a -> Pool Connection -> m a
+runSqlPool :: MonadBaseControl IO m => SqlPersist m a -> Pool Connection -> m a
 runSqlPool r pconn = withResource pconn $ runSqlConn r
 
-runSqlConn :: (MonadBaseControl IO m, MonadIO m) => SqlPersist m a -> Connection -> m a
+runSqlConn :: MonadBaseControl IO m => SqlPersist m a -> Connection -> m a
 runSqlConn (SqlPersist r) conn = do
     let getter = R.getStmt' conn
-    liftIO $ begin conn getter
+    liftBase $ begin conn getter
     x <- onException
             (runReaderT r conn)
-            (liftIO $ rollbackC conn getter)
-    liftIO $ commitC conn getter
+            (liftBase $ rollbackC conn getter)
+    liftBase $ commitC conn getter
     return x
 
-instance (MonadBaseControl IO m, MonadIO m, C.MonadThrow m, C.MonadUnsafeIO m) => PersistStore SqlPersist m where
+instance (C.MonadResource m, MonadLogger m) => PersistStore (SqlPersist m) where
+    type PersistMonadBackend (SqlPersist m) = R.SqlBackend
     insert val = do
         conn <- SqlPersist ask
-        let esql = insertSql conn (entityDB t) (map fieldDB $ entityFields t)
+        let esql = insertSql conn (entityDB t) (map fieldDB $ entityFields t) (entityID t)
         i <-
             case esql of
-                Left sql -> C.runResourceT $ R.withStmt sql vals C.$$ do
+                ISRSingle sql -> R.withStmt sql vals C.$$ do
                     x <- CL.head
                     case x of
                         Just [PersistInt64 i] -> return i
                         Nothing -> error $ "SQL insert did not return a result giving the generated ID"
                         Just vals' -> error $ "Invalid result from a SQL insert, got: " P.++ P.show vals'
-                Right (sql1, sql2) -> do
+                ISRInsertGet sql1 sql2 -> do
                     execute' sql1 vals
-                    C.runResourceT $ R.withStmt sql2 [] C.$$ do
+                    R.withStmt sql2 [] C.$$ do
                         Just [PersistInt64 i] <- CL.head
                         return i
         return $ Key $ PersistInt64 i
@@ -121,10 +129,12 @@ instance (MonadBaseControl IO m, MonadIO m, C.MonadThrow m, C.MonadUnsafeIO m) =
                 , escapeName conn (entityDB t)
                 , " SET "
                 , T.intercalate "," (map (go conn . fieldDB) $ entityFields t)
-                , " WHERE id=?"
+                , " WHERE "
+                , escapeName conn $ entityID t
+                , "=?"
                 ]
-        execute' sql $ map toPersistValue (toPersistFields val)
-                       `mappend` [unKey k]
+            vals = map toPersistValue (toPersistFields val) `mappend` [unKey k]
+        execute' sql vals
       where
         go conn x = escapeName conn x ++ "=?"
 
@@ -145,9 +155,12 @@ instance (MonadBaseControl IO m, MonadIO m, C.MonadThrow m, C.MonadUnsafeIO m) =
                 , cols
                 , " FROM "
                 , escapeName conn $ entityDB t
-                , " WHERE id=?"
+                , " WHERE "
+                , escapeName conn $ entityID t
+                , "=?"
                 ]
-        C.runResourceT $ R.withStmt sql [unKey k] C.$$ do
+            vals' = [unKey k]
+        R.withStmt sql vals' C.$$ do
             res <- CL.head
             case res of
                 Nothing -> return Nothing
@@ -164,12 +177,14 @@ instance (MonadBaseControl IO m, MonadIO m, C.MonadThrow m, C.MonadUnsafeIO m) =
         sql conn = concat
             [ "DELETE FROM "
             , escapeName conn $ entityDB t
-            , " WHERE id=?"
+            , " WHERE "
+            , escapeName conn $ entityID t
+            , "=?"
             ]
 
-insrepHelper :: (MonadIO m, PersistEntity val)
+insrepHelper :: (MonadIO m, PersistEntity val, MonadLogger m)
              => Text
-             -> Key SqlPersist val
+             -> Key val
              -> val
              -> SqlPersist m ()
 insrepHelper command (Key k) val = do
@@ -191,10 +206,12 @@ insrepHelper command (Key k) val = do
         ]
     vals = k : map toPersistValue (toPersistFields val)
 
-instance (MonadBaseControl IO m, C.MonadUnsafeIO m, MonadIO m, C.MonadThrow m) => PersistUnique SqlPersist m where
+instance (C.MonadResource m, MonadLogger m) => PersistUnique (SqlPersist m) where
     deleteBy uniq = do
         conn <- SqlPersist ask
-        execute' (sql conn) $ persistUniqueToValues uniq
+        let sql' = sql conn
+            vals = persistUniqueToValues uniq
+        execute' sql' vals
       where
         t = entityDef $ dummyFromUnique uniq
         go = map snd . persistUniqueToFieldNames
@@ -218,7 +235,8 @@ instance (MonadBaseControl IO m, C.MonadUnsafeIO m, MonadIO m, C.MonadThrow m) =
                 , " WHERE "
                 , sqlClause conn
                 ]
-        C.runResourceT $ R.withStmt sql (persistUniqueToValues uniq) C.$$ do
+            vals' = persistUniqueToValues uniq
+        R.withStmt sql vals' C.$$ do
             row <- CL.head
             case row of
                 Nothing -> return Nothing
@@ -234,120 +252,10 @@ instance (MonadBaseControl IO m, C.MonadUnsafeIO m, MonadIO m, C.MonadThrow m) =
         t = entityDef $ dummyFromUnique uniq
         toFieldNames' = map snd . persistUniqueToFieldNames
 
-dummyFromKey :: Key SqlPersist v -> v
+dummyFromKey :: KeyBackend R.SqlBackend v -> v
 dummyFromKey _ = error "dummyFromKey"
 
-{- FIXME
-<<<<<<< HEAD
-
-type Sql = Text
-
--- Bool indicates if the Sql is safe
-type CautiousMigration = [(Bool, Sql)]
-allSql :: CautiousMigration -> [Sql]
-allSql = map snd
-unsafeSql :: CautiousMigration -> [Sql]
-unsafeSql = allSql . filter fst
-safeSql :: CautiousMigration -> [Sql]
-safeSql = allSql . filter (not . fst)
-
-type Migration m = WriterT [Text] (WriterT CautiousMigration m) ()
-
-parseMigration :: Monad m => Migration m -> m (Either [Text] CautiousMigration)
-parseMigration =
-    liftM go . runWriterT . execWriterT
-  where
-    go ([], sql) = Right sql
-    go (errs, _) = Left errs
-
--- like parseMigration, but call error or return the CautiousMigration
-parseMigration' :: Monad m => Migration m -> m (CautiousMigration)
-parseMigration' m = do
-  x <- parseMigration m
-  case x of
-      Left errs -> error $ unpack $ unlines errs
-      Right sql -> return sql
-
-printMigration :: (MBCIO m, MonadIO m) => Migration (SqlPersist m) -> SqlPersist m ()
-printMigration m = do
-  mig <- parseMigration' m
-  mapM_ (liftIO . Data.Text.IO.putStrLn . flip snoc ';') (allSql mig)
-
-getMigration :: (MBCIO m, MonadIO m) => Migration (SqlPersist m) -> SqlPersist m [Sql]
-getMigration m = do
-  mig <- parseMigration' m
-  return $ allSql mig
-
-runMigration :: (MonadIO m, MBCIO m)
-             => Migration (SqlPersist m)
-             -> SqlPersist m ()
-runMigration m = runMigration' m False >> return ()
-
--- | Same as 'runMigration', but returns a list of the SQL commands executed
--- instead of printing them to stderr.
-runMigrationSilent :: (MBCIO m, MonadIO m)
-                   => Migration (SqlPersist m)
-                   -> SqlPersist m [Text]
-runMigrationSilent m = runMigration' m True
-
-runMigration'
-    :: (MBCIO m, MonadIO m)
-    => Migration (SqlPersist m)
-    -> Bool -- ^ is silent?
-    -> SqlPersist m [Text]
-runMigration' m silent = do
-    mig <- parseMigration' m
-    case unsafeSql mig of
-        []   -> mapM (executeMigrate silent) $ safeSql mig
-        errs -> error $ unpack $ concat
-            [ "\n\nDatabase migration: manual intervention required.\n"
-            , "The following actions are considered unsafe:\n\n"
-            , unlines $ map (\s -> "    " ++ s ++ ";") $ errs
-            ]
-
-runMigrationUnsafe :: (MBCIO m, MonadIO m)
-                   => Migration (SqlPersist m)
-                   -> SqlPersist m ()
-runMigrationUnsafe m = do
-    mig <- parseMigration' m
-    mapM_ (executeMigrate False) $ allSql mig
-
-executeMigrate :: MonadIO m => Bool -> Text -> SqlPersist m Text
-executeMigrate silent s = do
-    unless silent $ liftIO $ hPutStrLn stderr $ "Migrating: " ++ s
-    execute' s []
-    return s
-
-migrate :: (MonadIO m, MBCIO m, PersistEntity val)
-        => [EntityDef]
-        -> val
-        -> Migration (SqlPersist m)
-migrate allDefs val = do
-    conn <- lift $ lift $ SqlPersist ask
-    let getter = R.getStmt' conn
-    res <- liftIO $ migrateSql conn allDefs getter val
-    either tell (lift . tell) res
-
-updatePersistValue :: Update v -> PersistValue
-updatePersistValue (Update _ v _) = toPersistValue v
-
--- | Perform a database commit.
-commit :: MonadIO m => SqlPersist m ()
-commit = do
-    conn <- SqlPersist ask
-    let getter = R.getStmt' conn
-    liftIO $ commitC conn getter >> begin conn getter
-
--- | Perform a database rollback.
-rollback :: MonadIO m => SqlPersist m ()
-rollback = do
-    conn <- SqlPersist ask
-    let getter = R.getStmt' conn
-    liftIO $ rollbackC conn getter >> begin conn getter
-=======
--}
-
-dummyFromUnique :: Unique v b -> v
+dummyFromUnique :: Unique v -> v
 dummyFromUnique _ = error "dummyFromUnique"
 
 #if MIN_VERSION_monad_control(0, 3, 0)
@@ -365,6 +273,76 @@ show :: Show a => a -> Text
 show = pack . P.show
 
 
+-- | A 'Checkmark' should be used as a field type whenever a
+-- uniqueness constraint should guarantee that a certain kind of
+-- record may appear at most once, but other kinds of records may
+-- appear any number of times.
+--
+-- /NOTE:/ You need to mark any @Checkmark@ fields as @nullable@
+-- (see the following example).
+--
+-- For example, suppose there's a @Location@ entity that
+-- represents where a user has lived:
+--
+-- @
+-- Location
+--     user    UserId
+--     name    Text
+--     current Checkmark nullable
+--
+--     UniqueLocation user current
+-- @
+--
+-- The @UniqueLocation@ constraint allows any number of
+-- 'Inactive' @Location@s to be @current@.  However, there may be
+-- at most one @current@ @Location@ per user (i.e., either zero
+-- or one per user).
+--
+-- This data type works because of the way that SQL treats
+-- @NULL@able fields within uniqueness constraints.  The SQL
+-- standard says that @NULL@ values should be considered
+-- different, so we represent 'Inactive' as SQL @NULL@, thus
+-- allowing any number of 'Inactive' records.  On the other hand,
+-- we represent 'Active' as @TRUE@, so the uniqueness constraint
+-- will disallow more than one 'Active' record.
+--
+-- /Note:/ There may be DBMSs that do not respect the SQL
+-- standard's treatment of @NULL@ values on uniqueness
+-- constraints, please check if this data type works before
+-- relying on it.
+--
+-- The SQL @BOOLEAN@ type is used because it's the smallest data
+-- type available.  Note that we never use @FALSE@, just @TRUE@
+-- and @NULL@.  Provides the same behavior @Maybe ()@ would if
+-- @()@ was a valid 'PersistField'.
+data Checkmark = Active
+                 -- ^ When used on a uniqueness constraint, there
+                 -- may be at most one 'Active' record.
+               | Inactive
+                 -- ^ When used on a uniqueness constraint, there
+                 -- may be any number of 'Inactive' records.
+    deriving (Eq, Ord, Read, Show, Enum, Bounded)
+
+instance PersistField Checkmark where
+    toPersistValue Active   = PersistBool True
+    toPersistValue Inactive = PersistNull
+    fromPersistValue PersistNull         = Right Inactive
+    fromPersistValue (PersistBool True)  = Right Active
+    fromPersistValue (PersistBool False) =
+      Left "PersistField Checkmark: found unexpected FALSE value"
+    fromPersistValue other =
+      Left $ "PersistField Checkmark: unknown value " ++ show other
+    sqlType    _ = SqlBool
+    isNullable _ = True
+
+instance PathPiece Checkmark where
+    toPathPiece = show
+    fromPathPiece txt =
+      case reads (T.unpack txt) of
+        [(a, "")] -> Just a
+        _         -> Nothing
+
+
 -- $rawSql
 --
 -- Although it covers most of the useful cases, @persistent@'s
@@ -455,8 +433,8 @@ newtype Single a = Single {unSingle :: a}
 -- However, most common problems are mitigated by using the
 -- entity selection placeholder @??@, and you shouldn't see any
 -- error at all if you're not using 'Single'.
-rawSql :: (RawSql a, C.MonadUnsafeIO m, C.MonadThrow m, MonadIO m, MonadBaseControl IO m) =>
-          Text             -- ^ SQL statement, possibly with placeholders.
+rawSql :: (RawSql a, C.MonadResource m, MonadLogger m)
+       => Text             -- ^ SQL statement, possibly with placeholders.
        -> [PersistValue]   -- ^ Values to fill the placeholders.
        -> SqlPersist m [a]
 rawSql stmt = run
@@ -467,10 +445,10 @@ rawSql stmt = run
       x = getType run
       process = rawSqlProcessRow
 
-      withStmt' colSubsts = R.withStmt $ T.concat $
-                            makeSubsts colSubsts $
-                            T.splitOn placeholder stmt
+      withStmt' colSubsts params = do
+            R.withStmt sql params
           where
+            sql = T.concat $ makeSubsts colSubsts $ T.splitOn placeholder stmt
             placeholder = "??"
             makeSubsts (s:ss) (t:ts) = t : s : makeSubsts ss ts
             makeSubsts []     []     = []
@@ -487,7 +465,7 @@ rawSql stmt = run
       run params = do
         conn <- SqlPersist ask
         let (colCount, colSubsts) = rawSqlCols (escapeName conn) x
-        C.runResourceT $ withStmt' colSubsts params C.$$ firstRow colCount
+        withStmt' colSubsts params C.$$ firstRow colCount
 
       firstRow colCount = do
         mrow <- CL.head
@@ -530,7 +508,7 @@ instance PersistField a => RawSql (Single a) where
     rawSqlProcessRow _     = Left "RawSql (Single a): wrong number of columns."
 
 instance PersistEntity a => RawSql (Entity a) where
-    rawSqlCols escape = ((+1).length.entityFields &&& process) . entityDef . entityVal
+    rawSqlCols escape = ((+1) . length . entityFields &&& process) . entityDef . entityVal
         where
           process ed = (:[]) $
                        T.intercalate ", " $
@@ -548,6 +526,27 @@ instance PersistEntity a => RawSql (Entity a) where
                                           <*> fromPersistValues ent
     rawSqlProcessRow _ = Left "RawSql (Entity a): wrong number of columns."
 
+-- | Since 1.0.1.
+instance RawSql a => RawSql (Maybe a) where
+    rawSqlCols e = rawSqlCols e . extractMaybe
+    rawSqlColCountReason = rawSqlColCountReason . extractMaybe
+    rawSqlProcessRow cols
+      | all isNull cols = return Nothing
+      | otherwise       =
+        case rawSqlProcessRow cols of
+          Right v  -> Right (Just v)
+          Left msg -> Left $ "RawSql (Maybe a): not all columns were Null " ++
+                             "but the inner parser has failed.  Its message " ++
+                             "was \"" ++ msg ++ "\".  Did you apply Maybe " ++
+                             "to a tuple, perhaps?  The main use case for " ++
+                             "Maybe is to allow OUTER JOINs to be written, " ++
+                             "in which case 'Maybe (Entity v)' is used."
+      where isNull PersistNull = True
+            isNull _           = False
+
+extractMaybe :: Maybe a -> a
+extractMaybe = fromMaybe (error "Database.Persist.GenericSql.extractMaybe")
+
 instance (RawSql a, RawSql b) => RawSql (a, b) where
     rawSqlCols e x = rawSqlCols e (fst x) # rawSqlCols e (snd x)
         where (cnta, lsta) # (cntb, lstb) = (cnta + cntb, lsta P.++ lstb)
