@@ -1,3 +1,4 @@
+{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE PackageImports #-}
 {-# LANGUAGE CPP #-}
@@ -13,6 +14,8 @@ module Database.Persist.GenericSql.Internal
     , createSqlPool
     , mkColumns
     , Column (..)
+    , logSQL
+    , InsertSqlResult (..)
     ) where
 
 import qualified Data.Map as Map
@@ -23,17 +26,24 @@ import Data.Conduit.Pool
 import Database.Persist.Store
 import Control.Exception.Lifted (bracket)
 import Control.Monad.Trans.Control (MonadBaseControl)
-import Database.Persist.Util (nullable)
+import Database.Persist.Util (nullable, IsNullable(NotNullable))
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Monoid (Monoid, mappend, mconcat)
 import Database.Persist.EntityDef
 import qualified Data.Conduit as C
+import Language.Haskell.TH.Syntax (Q, Exp)
+import Control.Monad.Logger (logDebugS)
+import Data.Maybe (mapMaybe, listToMaybe)
+import Data.Int (Int64)
+
+data InsertSqlResult = ISRSingle Text
+                     | ISRInsertGet Text Text
 
 data Connection = Connection
     { prepare :: Text -> IO Statement
-    -- ^ table name, column names, either 1 or 2 statements to run
-    , insertSql :: DBName -> [DBName] -> Either Text (Text, Text)
+    -- ^ table name, column names, id name, either 1 or 2 statements to run
+    , insertSql :: DBName -> [DBName] -> DBName -> InsertSqlResult
     , stmtMap :: IORef (Map.Map Text Statement)
     , close :: IO ()
     , migrateSql :: forall v. PersistEntity v
@@ -50,7 +60,7 @@ data Connection = Connection
 data Statement = Statement
     { finalize :: IO ()
     , reset :: IO ()
-    , execute :: [PersistValue] -> IO ()
+    , execute :: [PersistValue] -> IO Int64
     , withStmt :: forall m. C.MonadResource m
                => [PersistValue]
                -> C.Source m [PersistValue]
@@ -106,8 +116,8 @@ mkColumns allDefs val =
     go fd p =
         Column
             (fieldDB fd)
-            (nullable $ fieldAttrs fd)
-            (sqlType p)
+            (nullable (fieldAttrs fd) /= NotNullable || entitySum t)
+            (maybe (sqlType p) SqlOther $ listToMaybe $ mapMaybe (T.stripPrefix "sqltype=") $ fieldAttrs fd)
             (def $ fieldAttrs fd)
             (maxLen $ fieldAttrs fd)
             (ref (fieldDB fd) (fieldType fd) (fieldAttrs fd))
@@ -187,3 +197,6 @@ tableColumn t s = go $ entityColumns t
         | x == s = ColumnDef x y z
         | otherwise = go rest
 -}
+
+logSQL :: Q Exp
+logSQL = [|\sql_foo params_foo -> $logDebugS (T.pack "SQL") $ T.pack $ show (sql_foo :: Text) ++ " " ++ show (params_foo :: [PersistValue])|]
