@@ -32,10 +32,13 @@ import Control.Monad.IO.Class
 import Control.Monad.Trans.Reader
 import Control.Monad.Trans.Writer
 import Control.Monad (liftM, unless)
-import Data.Text (Text, unpack, snoc)
+import Data.Text (Text, unpack, snoc, isPrefixOf, pack)
 import qualified Data.Text.IO
 import System.IO
 import Control.Monad.Logger (MonadLogger)
+import System.IO.Silently (hSilence)
+import System.IO (stderr)
+import Control.Monad.Trans.Control (liftBaseOp_)
 
 execute' :: (MonadIO m, MonadLogger m) => Text -> [PersistValue] -> SqlPersist m ()
 execute' = R.execute
@@ -88,7 +91,7 @@ runMigration m = runMigration' m False >> return ()
 runMigrationSilent :: (MBCIO m, MonadIO m, MonadLogger m)
                    => Migration (SqlPersist m)
                    -> SqlPersist m [Text]
-runMigrationSilent m = runMigration' m True
+runMigrationSilent m = liftBaseOp_ (hSilence [stderr]) $ runMigration' m True
 
 runMigration'
     :: (MBCIO m, MonadIO m, MonadLogger m)
@@ -98,7 +101,7 @@ runMigration'
 runMigration' m silent = do
     mig <- parseMigration' m
     case unsafeSql mig of
-        []   -> mapM (executeMigrate silent) $ safeSql mig
+        []   -> mapM (executeMigrate silent) $ sortMigrations $ safeSql mig
         errs -> error $ concat
             [ "\n\nDatabase migration: manual intervention required.\n"
             , "The following actions are considered unsafe:\n\n"
@@ -110,7 +113,7 @@ runMigrationUnsafe :: (MBCIO m, MonadIO m, MonadLogger m)
                    -> SqlPersist m ()
 runMigrationUnsafe m = do
     mig <- parseMigration' m
-    mapM_ (executeMigrate False) $ allSql mig
+    mapM_ (executeMigrate False) $ sortMigrations $ allSql mig
 
 executeMigrate :: (MonadIO m, MonadLogger m) => Bool -> Text -> SqlPersist m Text
 executeMigrate silent s = do
@@ -118,6 +121,16 @@ executeMigrate silent s = do
     execute' s []
     return s
 
+-- | Sort the alter DB statements so tables are created before constraints are
+-- added.
+sortMigrations :: [Sql] -> [Sql]
+sortMigrations x =
+    filter isCreate x ++ filter (not . isCreate) x
+  where
+    -- Note the use of lower-case e. This (hack) allows backends to explicitly
+    -- choose to have this special sorting applied.
+    isCreate t = pack "CREATe " `isPrefixOf` t
+
 migrate :: (MonadIO m, MBCIO m, PersistEntity val)
         => [EntityDef]
         -> val
