@@ -106,6 +106,7 @@ tokenize :: Text -> [Token]
 tokenize t
     | T.null t = []
     | "--" `T.isPrefixOf` t = [] -- Comment until the end of the line.
+    | "#" `T.isPrefixOf` t = [] -- Also comment to the end of the line, needed for a CPP bug (#110)
     | T.head t == '"' = quotes (T.tail t) id
     | T.head t == '(' = parens 1 (T.tail t) id
     | isSpace (T.head t) =
@@ -248,8 +249,10 @@ takeUniqs ps defs (n:rest)
         = Just $ UniqueDef
             (HaskellName n)
             (DBName $ psToDBName ps n)
-            (map (HaskellName &&& getDBName defs) rest)
+            (map (HaskellName &&& getDBName defs) fields)
+            attrs
   where
+    (fields,attrs) = break ("!" `T.isPrefixOf`) rest
     getDBName [] t = error $ "Unknown column in unique constraint: " ++ show t
     getDBName (d:ds) t
         | fieldHaskell d == HaskellName t = fieldDB d
