@@ -12,6 +12,7 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE RankNTypes #-}
 -- This is to test our assumption that OverlappingInstances is just for String
 #ifndef NO_OVERLAP
 {-# LANGUAGE OverlappingInstances #-}
@@ -30,6 +31,8 @@ module Database.Persist.Store
     , PersistFilter (..)
     , SomePersistField (..)
 
+    , ZT (..) -- ZonedTime wrapper
+
     , insertBy
     , getByValue
     , getJust
@@ -39,7 +42,8 @@ module Database.Persist.Store
     , checkUnique
     , DeleteCascade (..)
     , PersistException (..)
-    , Key (..)
+    , KeyBackend (..)
+    , Key
     , Entity (..)
 
       -- * Helpers
@@ -55,20 +59,15 @@ import qualified Prelude
 import Prelude hiding ((++), show)
 import Data.Monoid (mappend)
 import Data.Time (Day, TimeOfDay, UTCTime)
+import Data.Time.LocalTime (ZonedTime, zonedTimeToUTC, zonedTimeToLocalTime, zonedTimeZone)
 import Data.ByteString.Char8 (ByteString, unpack)
 import Control.Applicative
 import Data.Typeable (Typeable)
 import Data.Int (Int8, Int16, Int32, Int64)
 import Data.Word (Word8, Word16, Word32, Word64)
 
-#if MIN_VERSION_blaze_html(0,5,0)
 import Text.Blaze.Html
-import Text.Blaze.Internal (preEscapedText)
 import Text.Blaze.Html.Renderer.Text (renderHtml)
-#else
-import Text.Blaze (Html, preEscapedText)
-import Text.Blaze.Renderer.Text (renderHtml)
-#endif
 
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as TL
@@ -106,8 +105,30 @@ import Data.Text.Lazy (toStrict)
 import Data.Text.Lazy.Builder (toLazyText)
 
 import Control.Monad.Trans.Control (MonadBaseControl)
-import Control.Monad.Base (liftBase)
-import Control.Monad.IO.Class (MonadIO)
+import Control.Monad.Trans.Class (lift)
+import Control.Monad.IO.Class (MonadIO, liftIO)
+import Data.Monoid (Monoid)
+
+#if MIN_VERSION_conduit(1, 0, 0)
+import Data.Conduit.Internal (Pipe, ConduitM)
+#else
+import Data.Conduit (Pipe)
+#endif
+import Control.Monad.Logger (LoggingT)
+import Control.Monad.Trans.Identity ( IdentityT)
+import Control.Monad.Trans.List     ( ListT    )
+import Control.Monad.Trans.Maybe    ( MaybeT   )
+import Control.Monad.Trans.Error    ( ErrorT   )
+import Control.Monad.Trans.Reader   ( ReaderT  )
+import Control.Monad.Trans.Cont     ( ContT  )
+import Control.Monad.Trans.State    ( StateT   )
+import Control.Monad.Trans.Writer   ( WriterT  )
+import Control.Monad.Trans.RWS      ( RWST     )
+import Control.Monad.Trans.Resource ( ResourceT)
+
+import qualified Control.Monad.Trans.RWS.Strict    as Strict ( RWST   )
+import qualified Control.Monad.Trans.State.Strict  as Strict ( StateT )
+import qualified Control.Monad.Trans.Writer.Strict as Strict ( WriterT )
 
 data PersistException
   = PersistError T.Text -- ^ Generic Exception
@@ -122,6 +143,14 @@ instance E.Exception PersistException
 instance Error PersistException where
     strMsg = PersistError . T.pack
 
+-- | Avoid orphan instances.
+newtype ZT = ZT ZonedTime deriving (Show, Read, Typeable)
+
+instance Eq ZT where
+    ZT a /= ZT b = zonedTimeToLocalTime a /= zonedTimeToLocalTime b || zonedTimeZone a /= zonedTimeZone b
+instance Ord ZT where
+    ZT a `compare` ZT b = zonedTimeToUTC a `compare` zonedTimeToUTC b
+
 -- | A raw value which can be stored in any backend and can be marshalled to
 -- and from a 'PersistField'.
 data PersistValue = PersistText T.Text
@@ -132,6 +161,7 @@ data PersistValue = PersistText T.Text
                   | PersistDay Day
                   | PersistTimeOfDay TimeOfDay
                   | PersistUTCTime UTCTime
+                  | PersistZonedTime ZT
                   | PersistNull
                   | PersistList [PersistValue]
                   | PersistMap [(T.Text, PersistValue)]
@@ -157,6 +187,7 @@ instance A.ToJSON PersistValue where
     toJSON (PersistBool b) = A.Bool b
     toJSON (PersistTimeOfDay t) = A.String $ T.cons 't' $ show t
     toJSON (PersistUTCTime u) = A.String $ T.cons 'u' $ show u
+    toJSON (PersistZonedTime z) = A.String $ T.cons 'z' $ show z
     toJSON (PersistDay d) = A.String $ T.cons 'd' $ show d
     toJSON PersistNull = A.Null
     toJSON (PersistList l) = A.Array $ V.fromList $ map A.toJSON l
@@ -172,6 +203,7 @@ instance A.FromJSON PersistValue where
                            $ B64.decode $ TE.encodeUtf8 t
             Just ('t', t) -> fmap PersistTimeOfDay $ readMay t
             Just ('u', t) -> fmap PersistUTCTime $ readMay t
+            Just ('z', t) -> fmap PersistZonedTime $ readMay t
             Just ('d', t) -> fmap PersistDay $ readMay t
             Just ('o', t) -> either (fail "Invalid base64") (return . PersistObjectId)
                            $ B64.decode $ TE.encodeUtf8 t
@@ -197,13 +229,15 @@ instance A.FromJSON PersistValue where
 -- have different translations for these types.
 data SqlType = SqlString
              | SqlInt32
-             | SqlInteger -- ^ FIXME 8-byte integer; should be renamed SqlInt64
+             | SqlInt64
              | SqlReal
              | SqlBool
              | SqlDay
              | SqlTime
              | SqlDayTime
+             | SqlDayTimeZoned
              | SqlBlob
+             | SqlOther T.Text -- ^ a backend-specific name
     deriving (Show, Read, Eq, Typeable, Ord)
 
 -- | A value which can be marshalled to and from a 'PersistValue'.
@@ -225,6 +259,7 @@ instance PersistField String where
     fromPersistValue (PersistDay d) = Right $ Prelude.show d
     fromPersistValue (PersistTimeOfDay d) = Right $ Prelude.show d
     fromPersistValue (PersistUTCTime d) = Right $ Prelude.show d
+    fromPersistValue (PersistZonedTime (ZT z)) = Right $ Prelude.show z
     fromPersistValue PersistNull = Left "Unexpected null"
     fromPersistValue (PersistBool b) = Right $ Prelude.show b
     fromPersistValue (PersistList _) = Left "Cannot convert PersistList to String"
@@ -249,6 +284,7 @@ instance PersistField T.Text where
     fromPersistValue (PersistDay d) = Right $ show d
     fromPersistValue (PersistTimeOfDay d) = Right $ show d
     fromPersistValue (PersistUTCTime d) = Right $ show d
+    fromPersistValue (PersistZonedTime (ZT z)) = Right $ show z
     fromPersistValue PersistNull = Left "Unexpected null"
     fromPersistValue (PersistBool b) = Right $ show b
     fromPersistValue (PersistList _) = Left "Cannot convert PersistList to Text"
@@ -256,9 +292,14 @@ instance PersistField T.Text where
     fromPersistValue (PersistObjectId _) = Left "Cannot convert PersistObjectId to Text"
     sqlType _ = SqlString
 
+instance PersistField TL.Text where
+    toPersistValue = toPersistValue . TL.toStrict
+    fromPersistValue = fmap TL.fromStrict . fromPersistValue
+    sqlType _ = SqlString
+
 instance PersistField Html where
     toPersistValue = PersistText . TL.toStrict . renderHtml
-    fromPersistValue = fmap preEscapedText . fromPersistValue
+    fromPersistValue = fmap (preEscapedToMarkup :: T.Text -> Html) . fromPersistValue
     sqlType _ = SqlString
 
 instance PersistField Int where
@@ -267,7 +308,7 @@ instance PersistField Int where
     fromPersistValue x = Left $ "Expected Integer, received: " ++ show x
     sqlType x = case bitSize x of
                     32 -> SqlInt32
-                    _ -> SqlInteger
+                    _ -> SqlInt64
 
 instance PersistField Int8 where
     toPersistValue = PersistInt64 . fromIntegral
@@ -291,7 +332,7 @@ instance PersistField Int64 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i) = Right $ fromIntegral i
     fromPersistValue x = Left $ "Expected Integer, received: " ++ show x
-    sqlType _ = SqlInteger
+    sqlType _ = SqlInt64
 
 instance PersistField Word8 where
     toPersistValue = PersistInt64 . fromIntegral
@@ -309,13 +350,13 @@ instance PersistField Word32 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i) = Right $ fromIntegral i
     fromPersistValue x = Left $ "Expected Wordeger, received: " ++ show x
-    sqlType _ = SqlInteger
+    sqlType _ = SqlInt64
 
 instance PersistField Word64 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i) = Right $ fromIntegral i
     fromPersistValue x = Left $ "Expected Wordeger, received: " ++ show x
-    sqlType _ = SqlInteger
+    sqlType _ = SqlInt64
 
 instance PersistField Double where
     toPersistValue = PersistDouble
@@ -372,6 +413,20 @@ instance PersistField UTCTime where
     fromPersistValue x = Left $ "Expected UTCTime, received: " ++ show x
     sqlType _ = SqlDayTime
 
+instance PersistField ZonedTime where
+    toPersistValue = PersistZonedTime . ZT
+    fromPersistValue (PersistZonedTime (ZT z)) = Right z
+    fromPersistValue x@(PersistText t) =
+        case reads $ T.unpack t of
+            (z, _):_ -> Right z
+            _ -> Left $ "Expected ZonedTime, received " ++ show x
+    fromPersistValue x@(PersistByteString s) =
+        case reads $ unpack s of
+            (z, _):_ -> Right z
+            _ -> Left $ "Expected ZonedTime, received " ++ show x
+    fromPersistValue x = Left $ "Expected ZonedTime, received: " ++ show x
+    sqlType _ = SqlDayTimeZoned
+
 instance PersistField a => PersistField (Maybe a) where
     toPersistValue Nothing = PersistNull
     toPersistValue (Just a) = toPersistValue a
@@ -380,6 +435,11 @@ instance PersistField a => PersistField (Maybe a) where
     sqlType _ = sqlType (error "this is the problem" :: a)
     isNullable _ = True
 
+-- | Helper wrapper, equivalent to @Key (PersistEntityBackend val) val@.
+--
+-- Since 1.1.0
+type Key val = KeyBackend (PersistEntityBackend val) val
+
 -- | A single database entity. For example, if writing a blog application, a
 -- blog entry would be an entry, containing fields such as title and content.
 class PersistEntity val where
@@ -387,21 +447,24 @@ class PersistEntity val where
     data EntityField val :: * -> *
     persistFieldDef :: EntityField val typ -> FieldDef
 
-    type PersistEntityBackend val :: ((* -> *) -> * -> *)
+    type PersistEntityBackend val
 
     -- | Unique keys in existence on this entity.
-    data Unique val :: ((* -> *) -> * -> *) -> *
+    data Unique val
 
     entityDef :: val -> EntityDef
     toPersistFields :: val -> [SomePersistField]
     fromPersistValues :: [PersistValue] -> Either T.Text val
     halfDefined :: val
 
-    persistUniqueToFieldNames :: Unique val backend -> [(HaskellName, DBName)]
-    persistUniqueToValues :: Unique val backend -> [PersistValue]
-    persistUniqueKeys :: val -> [Unique val backend]
+    persistUniqueToFieldNames :: Unique val -> [(HaskellName, DBName)]
+    persistUniqueToValues :: Unique val -> [PersistValue]
+    persistUniqueKeys :: val -> [Unique val]
 
-    persistIdField :: EntityField val (Key (PersistEntityBackend val) val)
+    persistIdField :: EntityField val (Key val)
+
+    fieldLens :: EntityField val field
+              -> (forall f. Functor f => (field -> f field) -> Entity val -> f (Entity val))
 
 instance PersistField a => PersistField [a] where
     toPersistValue = PersistList . map toPersistValue
@@ -469,13 +532,13 @@ instance PersistField SomePersistField where
     fromPersistValue x = fmap SomePersistField (fromPersistValue x :: Either T.Text T.Text)
     sqlType (SomePersistField a) = sqlType a
 
-newtype Key (backend :: (* -> *) -> * -> *) entity = Key { unKey :: PersistValue }
+newtype KeyBackend backend entity = Key { unKey :: PersistValue }
     deriving (Show, Read, Eq, Ord, PersistField)
 
-instance A.ToJSON (Key backend entity) where
+instance A.ToJSON (KeyBackend backend entity) where
     toJSON (Key val) = A.toJSON val
 
-instance A.FromJSON (Key backend entity) where
+instance A.FromJSON (KeyBackend backend entity) where
     parseJSON = fmap Key . A.parseJSON
 
 -- | Datatype that represents an entity, with both its key and
@@ -509,59 +572,126 @@ instance A.FromJSON (Key backend entity) where
 -- Entity backend b)@), then you must you use @SELECT ??, ??
 -- WHERE ...@, and so on.
 data Entity entity =
-    Entity { entityKey :: Key (PersistEntityBackend entity) entity
+    Entity { entityKey :: Key entity
            , entityVal :: entity }
     deriving (Eq, Ord, Show, Read)
 
-class (MonadBaseControl IO m, MonadBaseControl IO (b m)) => PersistStore b m where
+class MonadIO m => PersistStore m where
+    type PersistMonadBackend m
 
     -- | Create a new record in the database, returning an automatically created
     -- key (in SQL an auto-increment id).
-    insert :: PersistEntity val => val -> b m (Key b val)
+    insert :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+           => val -> m (Key val)
+
+    -- | Same as 'insert', but doesn't return a @Key@.
+    insert_ :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+            => val -> m ()
+    insert_ val = insert val >> return ()
 
     -- | Create a new record in the database using the given key.
-    insertKey :: PersistEntity val => Key b val -> val -> b m ()
+    insertKey :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+              => Key val -> val -> m ()
 
     -- | Put the record in the database with the given key.
     -- Unlike 'replace', if a record with the given key does not
     -- exist then a new record will be inserted.
-    repsert :: PersistEntity val => Key b val -> val -> b m ()
+    repsert :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+            => Key val -> val -> m ()
 
     -- | Replace the record in the database with the given
     -- key. Note that the result is undefined if such record does
-    -- not exist, so you must use 'insertKey' or 'repsert' in
+    -- not exist, so you must use 'insertKey or 'repsert' in
     -- these cases.
-    replace :: PersistEntity val => Key b val -> val -> b m ()
+    replace :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+            => Key val -> val -> m ()
 
     -- | Delete a specific record by identifier. Does nothing if record does
     -- not exist.
-    delete :: PersistEntity val => Key b val -> b m ()
+    delete :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+           => Key val -> m ()
 
     -- | Get a record by identifier, if available.
-    get :: PersistEntity val => Key b val -> b m (Maybe val)
+    get :: (PersistMonadBackend m ~ PersistEntityBackend val, PersistEntity val)
+        => Key val -> m (Maybe val)
 
-class PersistStore b m => PersistUnique b m where
+#define DEF(T) { type PersistMonadBackend (T m) = PersistMonadBackend m; insert = lift . insert; insertKey k = lift . insertKey k; repsert k = lift . repsert k; replace k = lift . replace k; delete = lift . delete; get = lift . get }
+#define GO(T) instance (PersistStore m) => PersistStore (T m) where DEF(T)
+#define GOX(X, T) instance (X, PersistStore m) => PersistStore (T m) where DEF(T)
+
+GO(LoggingT)
+GO(IdentityT)
+GO(ListT)
+GO(MaybeT)
+GOX(Error e, ErrorT e)
+GO(ReaderT r)
+GO(ContT r)
+GO(StateT s)
+GO(ResourceT)
+GO(Pipe l i o u)
+#if MIN_VERSION_conduit(1, 0, 0)
+GO(ConduitM i o)
+#endif
+GOX(Monoid w, WriterT w)
+GOX(Monoid w, RWST r w s)
+GOX(Monoid w, Strict.RWST r w s)
+GO(Strict.StateT s)
+GOX(Monoid w, Strict.WriterT w)
+
+#undef DEF
+#undef GO
+#undef GOX
+
+-- | Queries against unique keys (other than the id).
+--
+-- Please read the general Persistent documentation to learn how to create
+-- Unique keys.
+-- SQL backends automatically create uniqueness constraints, but for MongoDB you must place a unique index on the field.
+class PersistStore m => PersistUnique m where
     -- | Get a record by unique key, if available. Returns also the identifier.
-    getBy :: (PersistEntityBackend val ~ b, PersistEntity val) => Unique val b -> b m (Maybe (Entity val))
+    getBy :: (PersistEntityBackend val ~ PersistMonadBackend m, PersistEntity val) => Unique val -> m (Maybe (Entity val))
 
     -- | Delete a specific record by unique key. Does nothing if no record
     -- matches.
-    deleteBy :: PersistEntity val => Unique val b -> b m ()
+    deleteBy :: (PersistEntityBackend val ~ PersistMonadBackend m, PersistEntity val) => Unique val -> m ()
 
     -- | Like 'insert', but returns 'Nothing' when the record
     -- couldn't be inserted because of a uniqueness constraint.
-    insertUnique :: (b ~ PersistEntityBackend val, PersistEntity val) => val -> b m (Maybe (Key b val))
+    insertUnique :: (PersistEntityBackend val ~ PersistMonadBackend m, PersistEntity val) => val -> m (Maybe (Key val))
     insertUnique datum = do
         isUnique <- checkUnique datum
         if isUnique then Just `liftM` insert datum else return Nothing
 
+#define DEF(T) { getBy = lift . getBy; deleteBy = lift . deleteBy; insertUnique = lift . insertUnique }
+#define GO(T) instance (PersistUnique m) => PersistUnique (T m) where DEF(T)
+#define GOX(X, T) instance (X, PersistUnique m) => PersistUnique (T m) where DEF(T)
+
+GO(LoggingT)
+GO(IdentityT)
+GO(ListT)
+GO(MaybeT)
+GOX(Error e, ErrorT e)
+GO(ReaderT r)
+GO(ContT r)
+GO(StateT s)
+GO(ResourceT)
+GO(Pipe l i o u)
+GOX(Monoid w, WriterT w)
+GOX(Monoid w, RWST r w s)
+GOX(Monoid w, Strict.RWST r w s)
+GO(Strict.StateT s)
+GOX(Monoid w, Strict.WriterT w)
+
+#undef DEF
+#undef GO
+#undef GOX
 
 
 -- | Insert a value, checking for conflicts with any unique constraints.  If a
 -- duplicate exists in the database, it is returned as 'Left'. Otherwise, the
--- new 'Key' is returned as 'Right'.
-insertBy :: (PersistEntity v, PersistStore b m, PersistUnique b m, b ~ PersistEntityBackend v)
-          => v -> b m (Either (Entity v) (Key b v))
+-- new 'Key is returned as 'Right'.
+insertBy :: (PersistEntity v, PersistStore m, PersistUnique m, PersistMonadBackend m ~ PersistEntityBackend v)
+          => v -> m (Either (Entity v) (Key v))
 insertBy val =
     go $ persistUniqueKeys val
   where
@@ -576,8 +706,8 @@ insertBy val =
 -- of a 'Unique' value. Returns a value matching /one/ of the unique keys. This
 -- function makes the most sense on entities with a single 'Unique'
 -- constructor.
-getByValue :: (PersistEntity v, PersistUnique b m, PersistEntityBackend v ~ b)
-           => v -> b m (Maybe (Entity v))
+getByValue :: (PersistEntity v, PersistUnique m, PersistEntityBackend v ~ PersistMonadBackend m)
+           => v -> m (Maybe (Entity v))
 getByValue val =
     go $ persistUniqueKeys val
   where
@@ -591,25 +721,29 @@ getByValue val =
 -- | curry this to make a convenience function that loads an associated model
 --   > foreign = belongsTo foeignId
 belongsTo ::
-  (PersistStore b m
+  (PersistStore m
   , PersistEntity ent1
-  , PersistEntity ent2) => (ent1 -> Maybe (Key b ent2)) -> ent1 -> b m (Maybe ent2)
+  , PersistEntity ent2
+  , PersistMonadBackend m ~ PersistEntityBackend ent2
+  ) => (ent1 -> Maybe (Key ent2)) -> ent1 -> m (Maybe ent2)
 belongsTo foreignKeyField model = case foreignKeyField model of
     Nothing -> return Nothing
     Just f -> get f
 
 -- | same as belongsTo, but uses @getJust@ and therefore is similarly unsafe
 belongsToJust ::
-  (PersistStore b m
+  (PersistStore m
   , PersistEntity ent1
-  , PersistEntity ent2) => (ent1 -> Key b ent2) -> ent1 -> b m ent2
+  , PersistEntity ent2
+  , PersistMonadBackend m ~ PersistEntityBackend ent2)
+  => (ent1 -> Key ent2) -> ent1 -> m ent2
 belongsToJust getForeignKey model = getJust $ getForeignKey model
 
 -- | Same as get, but for a non-null (not Maybe) foreign key
 --   Unsafe unless your database is enforcing that the foreign key is valid
-getJust :: (PersistStore b m, PersistEntity val, Show (Key b val)) => Key b val -> b m val
+getJust :: (PersistStore m, PersistEntity val, Show (Key val), PersistMonadBackend m ~ PersistEntityBackend val) => Key val -> m val
 getJust key = get key >>= maybe
-  (liftBase $ E.throwIO $ PersistForeignConstraintUnmet $ show key)
+  (liftIO $ E.throwIO $ PersistForeignConstraintUnmet $ show key)
   return
 
 
@@ -618,7 +752,7 @@ getJust key = get key >>= maybe
 --
 -- Returns 'True' if the entity would be unique, and could thus safely be
 -- 'insert'ed; returns 'False' on a conflict.
-checkUnique :: (PersistEntityBackend val ~ b, PersistEntity val, PersistUnique b m) => val -> b m Bool
+checkUnique :: (PersistEntityBackend val ~ PersistMonadBackend m, PersistEntity val, PersistUnique m) => val -> m Bool
 checkUnique val =
     go $ persistUniqueKeys val
   where
@@ -633,13 +767,13 @@ data PersistFilter = Eq | Ne | Gt | Lt | Ge | Le | In | NotIn
                    | BackendSpecificFilter T.Text
     deriving (Read, Show)
 
-class PersistEntity a => DeleteCascade a b m where
-    deleteCascade :: Key b a -> b m ()
+class (PersistStore m, PersistEntity a, PersistEntityBackend a ~ PersistMonadBackend m) => DeleteCascade a m where
+    deleteCascade :: Key a -> m ()
 
 instance PersistField PersistValue where
     toPersistValue = id
     fromPersistValue = Right
-    sqlType _ = SqlInteger -- since PersistValue should only be used like this for keys, which in SQL are Int64
+    sqlType _ = SqlInt64 -- since PersistValue should only be used like this for keys, which in SQL are Int64
 
 -- | Represents a value containing all the configuration options for a specific
 -- backend. This abstraction makes it easier to write code that can easily swap
